#include <Wire.h>
#include <LiquidCrystal_I2C.h>  // Library for LCD
#include <AccelStepper.h>       //Include accelstepper library

LiquidCrystal_I2C lcd(0x27, 16, 2);  // I2C address 0x27, 16 column and 2 rows

const int dirPin = 2;
const int stepPin = 3;

const int pl_incButtonPin = 4;   // Pin for the button to increase players
const int pl_decButtonPin = 5;   // Pin for the button to decrease players
const int cd_incButtonPin = 6;   // Pin for the button to increase cards
const int cd_decButtonPin = 7;   // Pin for the button to decrease cards
const int confirmButtonPin = 8;  // Pin to confirm selection

#define IN1_PIN 10
#define IN2_PIN 9
#define EN1_PIN 11

int pl_number = 1;  // Initial player number
int player_to_deal = 0;
int cards_to_deal = 1;
String* players;


int stepsPerRevolution = 200;

// Define motor interface type
#define motorInterfaceType 1

int currentPosition = 0;

AccelStepper myStepper(motorInterfaceType, stepPin, dirPin);

void setup() {
  lcd.init();       // initialize the lcd
  lcd.backlight();  // turn on the backlight

  pinMode(pl_incButtonPin, INPUT_PULLUP);   // Set pin for the increase button as INPUT_PULLUP
  pinMode(pl_decButtonPin, INPUT_PULLUP);   // Set pin for the decrease button as INPUT_PULLUP
  pinMode(cd_incButtonPin, INPUT_PULLUP);   // Set pin for the increase button as INPUT_PULLUP
  pinMode(cd_decButtonPin, INPUT_PULLUP);   // Set pin for the decrease button as INPUT_PULLUP
  pinMode(confirmButtonPin, INPUT_PULLUP);  //Set confirm pin as INPUT_PULLUP

  pinMode(EN1_PIN, OUTPUT);
  pinMode(IN1_PIN, OUTPUT);
  pinMode(IN2_PIN, OUTPUT);

  myStepper.setMaxSpeed(1000);
  myStepper.setAcceleration(800);
  myStepper.setSpeed(200);

  startUp();
  helloCards();
  selectNumberOfPlayers();
  fillPlayersList();
}

void loop() {
  selectPlayersAndCards();

}

void fillPlayersList() {
  players = new String[pl_number + 1];  // +1 for "All Players"
  players[0] = "All Players";

  for (int i = 1; i <= pl_number; i++) {
    players[i] = "Player " + String(i);
  }
}

//this is the startUp screen
void startUp() {
  lcd.clear();
  lcd.setCursor(4, 0);
  lcd.print("Welcome");
  delay(700);
  lcd.setCursor(7, 1);
  lcd.print("to");
  delay(300);
  lcd.clear();
  lcd.setCursor(4, 0);
  lcd.print("Welcome");
  lcd.setCursor(5, 1);
  lcd.print("to the");
  delay(500);
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("Automatic");
  delay(700);
  lcd.clear();
  lcd.setCursor(1, 0);
  lcd.print("Card");
  delay(500);
  lcd.print(" Shuffler");
  delay(500);
  lcd.setCursor(3, 1);
  lcd.print("and");
  delay(400);
  lcd.print(" Dealer");
  delay(2000);
}

//this loop allows players to make selections from the first screen
void selectNumberOfPlayers() {
  boolean selected = false;
  lcd.clear();
  while (!selected) {
    lcd.setCursor(0, 0);
    lcd.print(" Players number");
    lcd.setCursor(7, 1);
    lcd.print(String(pl_number));

    if (digitalRead(pl_incButtonPin) == LOW && pl_number < 8) {
      pl_number++;
      delay(200);
      lcd.clear();
    } else if (digitalRead(pl_decButtonPin) == LOW && pl_number > 1) {
      pl_number--;
      delay(200);
      lcd.clear();
    } else if (digitalRead(confirmButtonPin) == LOW) {
      lcd.clear();
      selected = true;
      delay(500);
    }
  }
}

void selectPlayersAndCards() {
  lcd.setCursor(0, 0);
  lcd.print("Deal to    cards");
  lcd.setCursor(0, 1);
  lcd.print(players[player_to_deal]);
  lcd.setCursor(14, 1);
  if (cards_to_deal < 10) {
    lcd.print(" ");
  }
  lcd.print(String(cards_to_deal));

  if (digitalRead(pl_incButtonPin) == LOW) {
    player_to_deal = (player_to_deal == pl_number) ? 0 : player_to_deal + 1;
    delay(200);
    lcd.clear();
  } else if (digitalRead(pl_decButtonPin) == LOW) {
    player_to_deal = (player_to_deal == 0) ? pl_number : player_to_deal - 1;
    delay(200);
    lcd.clear();
  } else if (digitalRead(cd_incButtonPin) == LOW) {
    cards_to_deal++;
    delay(200);
    lcd.clear();
  } else if (digitalRead(cd_decButtonPin) == LOW && cards_to_deal > 1) {
    cards_to_deal--;
    delay(200);
    lcd.clear();
  } else if (digitalRead(confirmButtonPin) == LOW) {
    moveStepper(player_to_deal, cards_to_deal);
    delay(200);
  }
}

void dealCards(int n) {

  digitalWrite(IN2_PIN, HIGH);  //Deal cards
  digitalWrite(IN1_PIN, LOW);
  delay(n * 525 - (n - 1) * 140);
  digitalWrite(IN2_PIN, LOW);  //Rotate backwards
  digitalWrite(IN1_PIN, HIGH);
  delay(400);
  digitalWrite(IN1_PIN, LOW);

}

//this part handles the dealing part with stepper
void moveStepper(int player, int n_card) {
  if (player == 0) {  //deal to all players
    myStepper.moveTo(0);
    myStepper.runToPosition();
    for (int p = 0; p < pl_number; p++) {
      int targetPosition = p * (stepsPerRevolution / pl_number);
      myStepper.moveTo(targetPosition);
      myStepper.runToPosition();
      dealCards(n_card);
    }
  } else {
    int targetPosition = (player - 1) * (stepsPerRevolution / pl_number);
    myStepper.moveTo(targetPosition);
    myStepper.runToPosition();
    dealCards(n_card);
  }
}

void helloCards() {
  analogWrite(EN1_PIN, 255);
  digitalWrite(IN1_PIN, HIGH);
  digitalWrite(IN2_PIN, LOW);
  delay(400);

  for (int i = 0; i < 3; i++) {
    digitalWrite(IN1_PIN, LOW);
    digitalWrite(IN2_PIN, HIGH);
    delay(150);

    digitalWrite(IN1_PIN, HIGH);
    digitalWrite(IN2_PIN, LOW);
    delay(200);
  }
  digitalWrite(IN2_PIN, LOW);
}